@@ -21,6 +21,12 @@
 import threading
 import devices as devicesModule
 import si
+import serial
+import time
+import logging
+import random
+import copy
+import serial.tools.list_ports
 
 class DeviceManager(object):
     """The DeviceManager manages devices.
@@ -32,7 +38,6 @@ class DeviceManager(object):
 
     _running = False
     _stopEvent = None
-    _stoppedEvent = None
     _thread = None
 
     _iterator = None
@@ -46,7 +51,6 @@ class DeviceManager(object):
         self._start()
 
     def _getIterator(self):
-        import random
         i = random.getrandbits(32)
         while True:
             yield i
@@ -69,8 +73,7 @@ class DeviceManager(object):
         if self._running:
             self.rawValues = self._thread.rawValues
         else:
-            import sys
-            sys.stderr.write("GetValuesThread not running. Can't update values.\n")
+            logging.warn("GetValuesThread not running. Can't update values.\n")
 
     def _getLinearFunction(self, value1, value2):
         """Return a linear function, which contains value1 and value2.
@@ -84,8 +87,7 @@ class DeviceManager(object):
         try:
             m = (value1[1] - value2[1]) / (value1[0] - value2[0])
         except ZeroDivisionError:
-            import sys
-            sys.stderr.write("ZeroDivisionError during slope computation.\nSetting slope to 1\n")
+            logging.warning("ZeroDivisionError while computing the slope. Setting slope to 1")
             m = 1
 
         c = value1[1] - m*value1[0]
@@ -140,7 +142,6 @@ class DeviceManager(object):
 
         """
 
-        import serial.tools.list_ports
         portsTuple = serial.tools.list_ports.comports()
         portsList = []
 
@@ -220,22 +221,27 @@ class DeviceManager(object):
 
         config["subDevices"] = {}
 
-        if isinstance(config["parent"], str):
-            device = eval("devicesModule." + config["deviceName"]).openRS232(config["parent"],
-                *config["args"], **config["kwargs"])
+        try:
+            if isinstance(config["parent"], str):
+                device = eval("devicesModule." + config["deviceName"]).openRS232(config["parent"],
+                    *config["args"], **config["kwargs"])
 
-        else:
-            assert self.devices[config["parent"]] # FIXME: use another error (e.g. WrongIDError)
+            else:
+                assert self.devices[config["parent"]] # FIXME: use another error (e.g. WrongIDError)
 
-            self.devices[config["parent"]].openDevice(eval("devicesModule." + config["deviceName"]),
-                input = config["inputNumber"], *config["args"], **config["kwargs"])
-            device = self.devices[config["parent"]].getDevice(input = config["inputNumber"])
-            self.configs[config["parent"]]["subDevices"][id] = config["inputNumber"]
+                self.devices[config["parent"]].openDevice(eval("devicesModule." + config["deviceName"]),
+                    input = config["inputNumber"], *config["args"], **config["kwargs"])
+                device = self.devices[config["parent"]].getDevice(input = config["inputNumber"])
+                self.configs[config["parent"]]["subDevices"][id] = config["inputNumber"]
 
-        self.devices[id] = device
-        self.configs[id] = config
+            self.devices[id] = device
+            self.configs[id] = config
 
-        return id
+            return id
+
+        except serial.serialutil.SerialException:
+            logging.error("Caught SerialException: Could not configure port: (5, 'Input/output error')")
+            return None
 
     @_pause
     def closeDevice(self, deviceID):
@@ -251,7 +257,6 @@ class DeviceManager(object):
         config = self.configs[deviceID]
 
         if isinstance(config["parent"], str):
-            import copy
             relCopy = copy.copy(config["subDevices"])
             # prevent RuntimeError: dictionary changed size during iteration
             
@@ -296,9 +301,7 @@ class DeviceManager(object):
             return self.rawValues[deviceID]
 
         except KeyError:
-            # import sys
-            # sys.stderr.write("KeyError in getLastRawValue (no value yet?)\n")
-            
+            logging.info("No value yet. Device: %s %s", deviceID, self.getDevice(deviceID))
             return devicesModule.NullValue()
 
     def getCalibratedLastRawValue(self, deviceID, calibration, unit=None):
@@ -373,10 +376,10 @@ class DeviceManager(object):
 
     def _start(self):
         """Start the DeviceManager."""
+
         if self._running == False:
             self._stopEvent = threading.Event()
-            self._stoppedEvent = threading.Event()
-            self._thread = _GetValuesThread(self._stopEvent, self._stoppedEvent,
+            self._thread = _GetValuesThread(self._stopEvent,
                 self.devices, self.configs)
             self._thread.start()
             self._running = True
@@ -386,10 +389,9 @@ class DeviceManager(object):
 
         if self._running == True:
             self._stopEvent.set()
-            self._stoppedEvent.wait() # wait until thread finishes
+            self._thread.join()
             self._thread = None
             self._stopEvent = None
-            self._stoppedEvent = None
             self._running = False
 
 
@@ -398,11 +400,10 @@ class _GetValuesThread(threading.Thread):
     configs = {}
     rawValues = {}
 
-    def __init__(self, stop_event, stopped_event, devices, configs):
+    def __init__(self, stop_event, devices, configs):
         threading.Thread.__init__(self)
         self.daemon = True
         self.stop_event = stop_event
-        self.stopped_event = stopped_event
 
         self.devices = devices
         self.configs = configs
@@ -411,14 +412,10 @@ class _GetValuesThread(threading.Thread):
             self.rawValues[key] = devicesModule.NullValue()
 
     def run(self):
-        import time
-
         while not self.stop_event.is_set():
             self.updateValue()
             time.sleep(0.2) # Prevent 100% CPU usage
 
-        self.stopped_event.set()
-
     def updateValue(self):
         # python3 incompatibility: iteritems
         for deviceID, config in self.configs.iteritems():
