@@ -20,6 +20,9 @@
 
 
 import si
+import serial
+import re
+import time
 
 deviceClassNames = ("TecpelDMM8061", "VoltcraftVC820", "XLS200", "KernPCB", "BS600")
 
@@ -31,8 +34,6 @@ class Device(object):
 
     """
 
-    import serial
-
     _ser = None # this is the serial device / connection
     _baudrate = 2400 # baudrate
     _rts = False # ready to send rs232 flag (used for setting up the rs232 connection)
@@ -92,8 +93,6 @@ class Device(object):
 
         """
 
-        import serial
-
         ser = serial.Serial()
         ser.port = port
         
@@ -191,8 +190,9 @@ class Value(object):
     _time = 0
 
     def __repr__(self):
-        return "< {classname}: displaydValue={!s}), unit={!s}, factor={!s} >".format(
-            self.getDisplayedValue(), self.getUnit(), self.getFactor())
+        return "< {classname}: displaydValue={!s}, factor={!s} >".format(
+            self.getDisplayedValue(), self.getFactor(),
+            classname = self.__class__.__name__)
 
     def __str__(self):
         return "{} {}".format(self.getDisplayedValue(), self.getFactor())
@@ -420,12 +420,13 @@ class TecpelDMM8061(Device):
             raise Exception
 
     def getRawValue(self):
-        import time
-        
         starttime = time.time()
         assert self.isAvailable()
 
         read = self._ser.read(26)
+
+        self._ser.flushInput()
+
         idList = []
         for i in read:
             idList.append(ord(i) >> 4)
@@ -695,7 +696,6 @@ class Balance(Device):
 class KernPCB(Balance):
     """This class represents the KernPCB Balance."""
 
-    import re
     _regex = re.compile(r"\s[\s-][\d\s\.]{10}\s.{3}\r\n")
 
     _baudrate = 9600
@@ -726,8 +726,6 @@ class KernPCB(Balance):
                 return ""
 
     def getRawValue(self):
-        import time
-
         starttime = time.time()
 
         assert self.isAvailable()
@@ -740,6 +738,7 @@ class KernPCB(Balance):
             # One value has 18 bytes, to make sure to get a complete one
             # (and not the second 1/2 of one, and the first 1/2 of another)
             # we read 35 bytes and match for one complete value
+            self._ser.flushInput()
 
             try:
                 result.string = self._regex.search(s).group()
@@ -757,6 +756,7 @@ class KernPCB(Balance):
             # One value has 18 bytes, to make sure to get a complete one
             # (and not the second 1/2 of one, and the first 1/2 of another)
             # we read 35 bytes and match for one complete value
+            self._ser.flushInput()
 
             try:
                 result.string = self._regex.search(s).group()
@@ -783,7 +783,6 @@ class KernPCB(Balance):
 class BS600(Balance):
     """This class represents the BS600 Balance."""
 
-    import re
     _stable = re.compile(r"[WCP][TC]ST[+-][\d\s\.]{7}.{4}")
     _all = re.compile(r"[WCP][TC][SU][TS][+-][\d\s\.]{7}.{4}")
 
@@ -814,8 +813,6 @@ class BS600(Balance):
                 return ""
     
     def getRawValue(self):
-        import time
-
         starttime = time.time()
 
         assert self.isAvailable()
@@ -826,6 +823,7 @@ class BS600(Balance):
         # One value has 18 bytes, to make sure to get a complete one
         # (and not the second 1/2 of one, and the first 1/2 of another)
         # we read 35 bytes and match for one complete value
+        self._ser.flushInput()
 
         try:
             if self._typeOfValue == "stable":
