@@ -18,10 +18,15 @@
 
 import sys
 import pkgutil
-import qr
+import time
+import os
+import tempfile
+import subprocess
 from PyQt4 import QtCore, QtGui, uic
+
 from devices.devicemanager import DeviceManager
-import devices.si as si
+from devices import si
+import ui_data.qr
 
 class NewDeviceDialog(QtGui.QDialog):
     """Dialog to add new devices."""
@@ -98,7 +103,6 @@ class SettingsDialog(QtGui.QDialog):
     def openFile(self):
         """Open a QFileDialog and save the path."""
 
-        import os
         popup = QtGui.QFileDialog()
         self.path.setText(popup.getOpenFileName(self, self.tr("Search Office"), os.path.expanduser("~"), ""))
 
@@ -115,10 +119,12 @@ class DisplayWidget(QtGui.QWidget):
     twoValueCalibration = (0.0, 0.0), (1.0, 1.0)
     slopeInterceptCalibration = 1.0, 0.0
 
-    is1PrefixIndex = 0 # QComboBox Index
-    should1PrefixIndex = 0 # QComboBox Index
-    is2PrefixIndex = 0 # QComboBox Index
-    should2PrefixIndex = 0 # QComboBox Index
+    siNames = si.getSiNames("normal")
+
+    is1PrefixIndex = siNames.index("") # QComboBox Index
+    should1PrefixIndex = siNames.index("") # QComboBox Index
+    is2PrefixIndex = siNames.index("") # QComboBox Index
+    should2PrefixIndex = siNames.index("") # QComboBox Index
 
     is1 = 0
     should1 = 0
@@ -168,7 +174,7 @@ class DisplayWidget(QtGui.QWidget):
         # Add prefixes to combobox
         for i in [popup.is1Prefix, popup.should1Prefix, popup.is2Prefix, popup.should2Prefix]:
             i.clear()
-            i.addItems(si.getSiNames())
+            i.addItems(self.siNames)
 
         popup.is1Prefix.setCurrentIndex(self.is1PrefixIndex)
         popup.should1Prefix.setCurrentIndex(self.should1PrefixIndex)
@@ -248,6 +254,11 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.timer.timeout.connect(self.update)
         self.timer.start(50)
 
+        self.finished.connect(self._finish)
+
+    def _finish(self):
+        self.timer.stop()
+
     def update(self):
         self.twoValueCalibration = ((self.is1.value() * si.getFactor(str(self.is1Prefix.currentText())),
                 self.should1.value()  * si.getFactor(str(self.should1Prefix.currentText()))),
@@ -257,10 +268,9 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.slopeInterceptCalibration = self.slope.value(), self.intercept.value()
         
         if self.slopeInterceptButton.isChecked():
-            self = slopeInterceptCalibration
+            self.calibration = self.slopeInterceptCalibration
         elif self.valuesButton.isChecked():
-            self = twoValueCalibration
-
+            self.calibration = self.twoValueCalibration
 
         crv = self.dm.getCalibratedLastRawValue(self.deviceID, self.calibration, self.unit)
         self.calibratedLabel.setText(str(crv.getDisplayedValue() * crv.getFactor()))
@@ -307,22 +317,25 @@ class DeviceSettingsDialog(QtGui.QDialog):
             self.unit.text())
 
     def load(self):
+        allItems = [self.is1Prefix.itemText(i) for i in range(self.is1Prefix.count())]
+        emptyStringIndex = allItems.index("")
+
         self.is1.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "is1", 0).toInt()[0])
         self.should1.setValue(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "should1", 1).toInt()[0])
+            self.slotComboBox.currentText() + "should1", 0).toInt()[0])
         self.is2.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "is2", 1).toInt()[0])
         self.should2.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "should2", 1).toInt()[0])
         self.is1Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "is1PrefixIndex", "").toInt()[0])
+            self.slotComboBox.currentText() + "is1PrefixIndex", emptyStringIndex).toInt()[0])
         self.should1Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "should1PrefixIndex", "").toInt()[0])
+            self.slotComboBox.currentText() + "should1PrefixIndex", emptyStringIndex).toInt()[0])
         self.is2Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "is2PrefixIndex", "").toInt()[0])
+            self.slotComboBox.currentText() + "is2PrefixIndex", emptyStringIndex).toInt()[0])
         self.should2Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "should2PrefixIndex", "").toInt()[0])
+            self.slotComboBox.currentText() + "should2PrefixIndex", emptyStringIndex).toInt()[0])
 
         self.slope.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "slope", 1).toInt()[0])
@@ -411,9 +424,6 @@ class MainWindow(QtGui.QMainWindow):
     def openLog(self):
         """Open last log with office."""
         
-        import subprocess
-        import os
-
         if self.pathToLogFile:
             subprocess.call('"' + self.officePath + '"' + ' ' + 
                 '"' + self.pathToLogFile + '"', shell=True)
@@ -447,40 +457,41 @@ class MainWindow(QtGui.QMainWindow):
 
                 if xls200Popup.result():
                     xls200ID = self.dm.openDevice(device, port)
-                    
-                    if xls200Popup.subdevice1ComboBox.currentIndex() != 0:
-                        sub1 = unicode(xls200Popup.subdevice1ComboBox.currentText())
-                        deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
-                        sub1Widget = DisplayWidget(deviceID, self.dm)
-                        self.verticalLayout.addWidget(sub1Widget)
-                        self.displayWidgets[deviceID] = sub1Widget
 
-                    if xls200Popup.subdevice2ComboBox.currentIndex() != 0:
-                        sub2 = unicode(xls200Popup.subdevice2ComboBox.currentText())
-                        deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
-                        sub2Widget = DisplayWidget(deviceID, self.dm)
-                        self.verticalLayout.addWidget(sub2Widget)
-                        self.displayWidgets[deviceID] = sub2Widget
+                    if xls200ID:
+                        
+                        if xls200Popup.subdevice1ComboBox.currentIndex() != 0:
+                            sub1 = unicode(xls200Popup.subdevice1ComboBox.currentText())
+                            deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
+                            sub1Widget = DisplayWidget(deviceID, self.dm)
+                            self.verticalLayout.addWidget(sub1Widget)
+                            self.displayWidgets[deviceID] = sub1Widget
 
-                    if xls200Popup.subdevice3ComboBox.currentIndex() != 0:
-                        sub3 = unicode(xls200Popup.subdevice3ComboBox.currentText())
-                        deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
-                        sub3Widget = DisplayWidget(deviceID, self.dm)
-                        self.verticalLayout.addWidget(sub3Widget)
-                        self.displayWidgets[deviceID] = sub3Widget
+                        if xls200Popup.subdevice2ComboBox.currentIndex() != 0:
+                            sub2 = unicode(xls200Popup.subdevice2ComboBox.currentText())
+                            deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
+                            sub2Widget = DisplayWidget(deviceID, self.dm)
+                            self.verticalLayout.addWidget(sub2Widget)
+                            self.displayWidgets[deviceID] = sub2Widget
+
+                        if xls200Popup.subdevice3ComboBox.currentIndex() != 0:
+                            sub3 = unicode(xls200Popup.subdevice3ComboBox.currentText())
+                            deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
+                            sub3Widget = DisplayWidget(deviceID, self.dm)
+                            self.verticalLayout.addWidget(sub3Widget)
+                            self.displayWidgets[deviceID] = sub3Widget
 
             else:
                 deviceID = self.dm.openDevice(device, port)
-                deviceWidget = DisplayWidget(deviceID, self.dm)
-                self.verticalLayout.addWidget(deviceWidget)
-                self.displayWidgets[deviceID] = deviceWidget
+
+                if deviceID:
+                    deviceWidget = DisplayWidget(deviceID, self.dm)
+                    self.verticalLayout.addWidget(deviceWidget)
+                    self.displayWidgets[deviceID] = deviceWidget
 
     def startStopLogging(self):
         """Start/Stop logging."""
 
-        import time
-        import tempfile
-
         if self.log == False:
             if self.tmpfile:
                 popup = DoReallyDialog(self.tr("Overwrite last log"),
@@ -503,8 +514,6 @@ class MainWindow(QtGui.QMainWindow):
     def saveLog(self):
         """Save last log to file."""
 
-        import os
-
         if self.tmpfile:
             popup = QtGui.QFileDialog()
             filename = QtGui.QFileDialog.getSaveFileName(self, self.tr("Save file"),
@@ -527,7 +536,6 @@ class MainWindow(QtGui.QMainWindow):
     def update(self):
         """Update the displayWidgets and, when logging enabled log."""
         
-        import time
         # python3 incompatibility: .iteritems()
         # Delete unnecessary widgets
         deviceIDsToBeDeleted = []
@@ -538,12 +546,8 @@ class MainWindow(QtGui.QMainWindow):
         self.dm.closeEmptyMultiboxDevices()
 
         for i in deviceIDsToBeDeleted:
-            try:
-                self.displayWidgets[i].delete()
-                del(self.displayWidgets[i])
-            except KeyError:
-                # Some devices don't have widgets (e.g. xls200), so there are no widgets to be deleted
-                pass
+            self.displayWidgets[i].delete()
+            del(self.displayWidgets[i])
 
         # update widgets
         for deviceID, widget in self.displayWidgets.iteritems():
@@ -580,7 +584,7 @@ class App(QtGui.QApplication):
     def byebye(self):
         self.exit(0)
 
-if __name__ == "__main__":
+def main():
     app = App(sys.argv)
 
     QtCore.QCoreApplication.setOrganizationName("Lausen")
@@ -599,4 +603,7 @@ if __name__ == "__main__":
         app.installTranslator(translator)
 
     app.setup()
-    app.exec_()
\ No newline at end of file
+    app.exec_()
+
+if __name__ == "__main__":
+    main()
\ No newline at end of file
